@@ -8,10 +8,12 @@ from pathlib import Path
 
 from aind_data_schema.procedures import (
     Anaesthetic,
+    CoordinateReferenceLocation,
     Craniotomy,
     CraniotomyType,
     FiberImplant,
     Headframe,
+    InjectionMaterial,
     IontophoresisInjection,
     NanojectInjection,
     OphysProbe,
@@ -140,12 +142,17 @@ class Examples:
         level="Select...",
     )
 
+    bregma_reference = CoordinateReferenceLocation.BREGMA
+    lambda_reference = CoordinateReferenceLocation.LAMBDA
+
     expected_probe1 = OphysProbe.construct(
         name="Probe A",
         stereotactic_coordinate_ml=-3.3,
         stereotactic_coordinate_ap=-1.6,
         stereotactic_coordinate_dv=4.2,
         angle=0.0,
+        stereotactic_coordinate_reference=bregma_reference,
+        bregma_to_lambda_distance=4.0,
     )
 
     expected_probe2 = OphysProbe.construct(
@@ -154,9 +161,68 @@ class Examples:
         stereotactic_coordinate_ap=-3.05,
         stereotactic_coordinate_dv=4.2,
         angle=0.0,
+        bregma_to_lambda_distance=4.0,
+        stereotactic_coordinate_reference=bregma_reference,
+    )
+
+    expected_inj_materials_1 = InjectionMaterial.construct(
+        full_genome_name="Premixied &quot;\u200bdL+Cre&quot;"
+    )
+    expected_inj_materials_2 = InjectionMaterial.construct(
+        full_genome_name="DIO-ChrimsonR"
     )
 
     expected_subject_procedures1 = [
+        # from list item 1
+        NanojectInjection.construct(
+            start_date=datetime.date(2022, 12, 6),
+            end_date=datetime.date(2022, 12, 6),
+            experimenter_full_name="NSB-187",
+            iacuc_protocol="2115",
+            animal_weight_prior=19.1,
+            animal_weight_post=19.2,
+            anaesthesia=expected_inj_anaesthetic,
+            notes=None,
+            injection_duration=5,
+            recovery_time=None,
+            workstation_id="SWS 3",
+            instrument_id="Select...",
+            injection_hemisphere=Side.LEFT,
+            injection_coordinate_ml=-3.3,
+            injection_coordinate_ap=-1.6,
+            injection_coordinate_depth=4.3,
+            injection_angle=0.0,
+            injection_type="Nanoject (Pressure)",
+            injection_volume=400.0,
+            injection_materials=expected_inj_materials_1,
+            injection_coordinate_reference=bregma_reference,
+            bregma_to_lambda_distance=4.0,
+        ),
+        IontophoresisInjection.construct(
+            start_date=None,
+            end_date=None,
+            experimenter_full_name="NSB-187",
+            iacuc_protocol="2115",
+            animal_weight_prior=19.6,
+            animal_weight_post=19.6,
+            anaesthesia=expected_inj_anaesthetic,
+            notes=None,
+            injection_materials=expected_inj_materials_2,
+            injection_duration=4.0,
+            recovery_time=None,
+            workstation_id=None,
+            instrument_id="Select...",
+            injection_hemisphere=Side.LEFT,
+            injection_coordinate_ml=-0.6,
+            injection_coordinate_ap=-3.05,
+            injection_coordinate_depth=4.3,
+            injection_angle=None,
+            injection_type="Iontophoresis",
+            injection_current=None,
+            alternating_current="7/7",
+            bregma_to_lambda_distance=4.0,
+            injection_coordinate_reference=bregma_reference,
+        ),
         Headframe.construct(
             start_date=datetime.date(2022, 12, 6),
             end_date=datetime.date(2022, 12, 6),
@@ -171,135 +237,36 @@ class Examples:
             well_part_number=None,
             well_type=None,
         ),
-        Headframe.construct(
+        Craniotomy.construct(
             start_date=datetime.date(2022, 12, 6),
             end_date=datetime.date(2022, 12, 6),
             experimenter_full_name="NSB-187",
             iacuc_protocol="2115",
             animal_weight_prior=None,
             animal_weight_post=None,
-            notes=None,
             anaesthesia=expected_hp_anaesthetic,
-            headframe_type="CAM-style",
-            headframe_part_number="0160-100-10 Rev A",
-            well_part_number=None,
-            well_type="CAM-style",
-        ),
-        Headframe.construct(
-            start_date=datetime.date(2022, 12, 6),
-            end_date=datetime.date(2022, 12, 6),
-            experimenter_full_name="NSB-187",
-            iacuc_protocol="2115",
-            animal_weight_prior=None,
-            animal_weight_post=None,
-            notes=None,
-            anaesthesia=expected_hp_anaesthetic,
-            headframe_type="NGC-style",
-            headframe_part_number="0160-100-10",
-            well_part_number="0160-200-20",
-            well_type="Mesoscope-style",
-        ),
-        Headframe.construct(
-            start_date=datetime.date(2022, 12, 6),
-            end_date=datetime.date(2022, 12, 6),
-            experimenter_full_name="NSB-187",
-            iacuc_protocol="2115",
-            animal_weight_prior=None,
-            animal_weight_post=None,
-            notes=None,
-            anaesthesia=expected_hp_anaesthetic,
-            headframe_type="Neuropixel-style",
-            headframe_part_number="0160-100-10",
-            well_part_number="0160-200-36",
-            well_type="Neuropixel-style",
-        ),
-        Headframe.construct(
-            start_date=datetime.date(2022, 12, 6),
-            end_date=datetime.date(2022, 12, 6),
-            experimenter_full_name="NSB-187",
-            iacuc_protocol="2115",
-            animal_weight_prior=None,
-            animal_weight_post=None,
-            notes=None,
-            anaesthesia=expected_hp_anaesthetic,
-            headframe_type="WHC #42",
-            headframe_part_number="42",
-            well_part_number="0160-200-36",
-            well_type="Neuropixel-style",
-        ),
-        Headframe.construct(
-            start_date=datetime.date(2022, 12, 6),
-            end_date=datetime.date(2022, 12, 6),
-            experimenter_full_name="NSB-187",
-            iacuc_protocol="2115",
-            animal_weight_prior=None,
-            animal_weight_post=None,
-            notes=None,
-            anaesthesia=expected_hp_anaesthetic,
-            headframe_type="NGC-style",
-            headframe_part_number="0160-100-10",
-            well_part_number=None,
-            well_type=None,
-        ),
-        Headframe.construct(
-            start_date=datetime.date(2022, 12, 6),
-            end_date=datetime.date(2022, 12, 6),
-            experimenter_full_name="NSB",
-            iacuc_protocol="2115",
-            animal_weight_prior=None,
-            animal_weight_post=None,
-            notes=None,
-            anaesthesia=expected_hp_anaesthetic,
-            headframe_type=None,
-            headframe_part_number=None,
-            well_part_number=None,
-            well_type=None,
-        ),
-        NanojectInjection.construct(
-            start_date=datetime.date(2022, 12, 6),
-            end_date=datetime.date(2022, 12, 6),
-            experimenter_full_name="NSB-187",
-            iacuc_protocol="2115",
-            animal_weight_prior=19.1,
-            animal_weight_post=19.2,
-            anaesthesia=expected_inj_anaesthetic,
-            notes=None,
-            injection_materials=None,
-            injection_duration=5,
-            recovery_time=None,
+            craniotomy_type=CraniotomyType.VISCTX,
+            craniotomy_hemisphere="Right",
+            craniotomy_coordinates_ml=2.8,
+            craniotomy_coordinates_ap=1.3,
+            craniotomy_size=5.0,
+            dura_removed=True,
             workstation_id="SWS 3",
-            instrument_id="Select...",
-            injection_hemisphere=Side.LEFT,
-            injection_coordinate_ml=-3.3,
-            injection_coordinate_ap=-1.6,
-            injection_coordinate_depth=4.3,
-            injection_angle=0.0,
-            injection_type="Nanoject (Pressure)",
-            injection_volume=400.0,
+            notes=None,
+            bregma_to_lambda_distance=4.0,
+            craniotomy_coordinates_reference=lambda_reference,
         ),
-        IontophoresisInjection.construct(
-            start_date=None,
-            end_date=None,
+        FiberImplant.construct(
+            start_date=datetime.date(2022, 12, 6),
+            end_date=datetime.date(2022, 12, 6),
             experimenter_full_name="NSB-187",
             iacuc_protocol="2115",
-            animal_weight_prior=19.6,
-            animal_weight_post=19.6,
-            anaesthesia=expected_inj_anaesthetic,
+            animal_weight_prior=None,
+            animal_weight_post=None,
             notes=None,
-            injection_materials=None,
-            injection_duration=4,
-            recovery_time=None,
-            workstation_id=None,
-            instrument_id="Select...",
-            injection_hemisphere=Side.LEFT,
-            injection_coordinate_ml=-0.6,
-            injection_coordinate_ap=-3.05,
-            injection_coordinate_depth=4.3,
-            injection_angle=None,
-            injection_type="Iontophoresis",
-            injection_current=None,
-            alternating_current="7/7",
+            probes=[expected_probe1],
         ),
+        # from list item 2
         IontophoresisInjection.construct(
             start_date=datetime.date(2022, 12, 6),
             end_date=datetime.date(2022, 12, 6),
@@ -322,6 +289,7 @@ class Examples:
             injection_type="Iontophoresis",
             injection_current=5.0,
             alternating_current="7/7",
+            injection_coordinate_reference=lambda_reference,
         ),
         NanojectInjection.construct(
             start_date=datetime.date(2022, 12, 6),
@@ -344,6 +312,53 @@ class Examples:
             injection_angle=None,
             injection_type="Nanoject (Pressure)",
             injection_volume=400.0,
+            injection_coordinate_reference=bregma_reference,
+        ),
+        Craniotomy.construct(
+            start_date=datetime.date(2022, 12, 6),
+            end_date=datetime.date(2022, 12, 6),
+            experimenter_full_name="NSB-187",
+            iacuc_protocol="2115",
+            animal_weight_prior=None,
+            animal_weight_post=None,
+            anaesthesia=expected_hp_anaesthetic,
+            craniotomy_type=CraniotomyType.WHC,
+            craniotomy_hemisphere="Center",
+            craniotomy_coordinates_ml=2.8,
+            craniotomy_coordinates_ap=1.3,
+            craniotomy_size=5.0,
+            dura_removed=False,
+            workstation_id="SWS 3",
+            notes=None,
+        ),
+        Headframe.construct(
+            start_date=datetime.date(2022, 12, 6),
+            end_date=datetime.date(2022, 12, 6),
+            experimenter_full_name="NSB-187",
+            iacuc_protocol="2115",
+            animal_weight_prior=None,
+            animal_weight_post=None,
+            notes=None,
+            anaesthesia=expected_hp_anaesthetic,
+            headframe_type="CAM-style",
+            headframe_part_number="0160-100-10 Rev A",
+            well_part_number=None,
+            well_type="CAM-style",
+        ),
+        # from list item 3
+        Headframe.construct(
+            start_date=datetime.date(2022, 12, 6),
+            end_date=datetime.date(2022, 12, 6),
+            experimenter_full_name="NSB-187",
+            iacuc_protocol="2115",
+            animal_weight_prior=None,
+            animal_weight_post=None,
+            notes=None,
+            anaesthesia=expected_hp_anaesthetic,
+            headframe_type="NGC-style",
+            headframe_part_number="0160-100-10",
+            well_part_number="0160-200-20",
+            well_type="Mesoscope-style",
         ),
         IontophoresisInjection.construct(
             start_date=datetime.date(2022, 12, 6),
@@ -367,40 +382,18 @@ class Examples:
             injection_type="Iontophoresis",
             injection_current=5.0,
             alternating_current="7/7",
+            injection_coordinate_reference=bregma_reference,
+            bregma_to_lambda_distance=4.0,
         ),
-        Craniotomy.construct(
+        FiberImplant.construct(
             start_date=datetime.date(2022, 12, 6),
             end_date=datetime.date(2022, 12, 6),
             experimenter_full_name="NSB-187",
             iacuc_protocol="2115",
             animal_weight_prior=None,
             animal_weight_post=None,
-            anaesthesia=expected_hp_anaesthetic,
-            craniotomy_type=CraniotomyType.VISCTX,
-            craniotomy_hemisphere="Right",
-            craniotomy_coordinates_ml=2.8,
-            craniotomy_coordinates_ap=1.3,
-            craniotomy_size=5.0,
-            dura_removed=True,
-            workstation_id="SWS 3",
-            notes=None,
-        ),
-        Craniotomy.construct(
-            start_date=datetime.date(2022, 12, 6),
-            end_date=datetime.date(2022, 12, 6),
-            experimenter_full_name="NSB-187",
-            iacuc_protocol="2115",
-            animal_weight_prior=None,
-            animal_weight_post=None,
-            anaesthesia=expected_hp_anaesthetic,
-            craniotomy_type=CraniotomyType.WHC,
-            craniotomy_hemisphere="Center",
-            craniotomy_coordinates_ml=2.8,
-            craniotomy_coordinates_ap=1.3,
-            craniotomy_size=5.0,
-            dura_removed=False,
-            workstation_id="SWS 3",
             notes=None,
+            probes=[expected_probe1, expected_probe2],
         ),
         Craniotomy.construct(
             start_date=datetime.date(2022, 12, 6),
@@ -418,6 +411,22 @@ class Examples:
             dura_removed=None,
             workstation_id="SWS 3",
             notes=None,
+            bregma_to_lambda_distance=4.0,
+        ),
+        # from list item 4
+        Headframe.construct(
+            start_date=datetime.date(2022, 12, 6),
+            end_date=datetime.date(2022, 12, 6),
+            experimenter_full_name="NSB-187",
+            iacuc_protocol="2115",
+            animal_weight_prior=None,
+            animal_weight_post=None,
+            notes=None,
+            anaesthesia=expected_hp_anaesthetic,
+            headframe_type="Neuropixel-style",
+            headframe_part_number="0160-100-10",
+            well_part_number="0160-200-36",
+            well_type="Neuropixel-style",
         ),
         Craniotomy.construct(
             start_date=datetime.date(2022, 12, 6),
@@ -436,6 +445,37 @@ class Examples:
             workstation_id="SWS 3",
             notes=None,
         ),
+        # from list item 5
+        Headframe.construct(
+            start_date=datetime.date(2022, 12, 6),
+            end_date=datetime.date(2022, 12, 6),
+            experimenter_full_name="NSB-187",
+            iacuc_protocol="2115",
+            animal_weight_prior=None,
+            animal_weight_post=None,
+            notes=None,
+            anaesthesia=expected_hp_anaesthetic,
+            headframe_type="WHC #42",
+            headframe_part_number="42",
+            well_part_number="0160-200-36",
+            well_type="Neuropixel-style",
+        ),
+        # from list item 6
+        Headframe.construct(
+            start_date=datetime.date(2022, 12, 6),
+            end_date=datetime.date(2022, 12, 6),
+            experimenter_full_name="NSB-187",
+            iacuc_protocol="2115",
+            animal_weight_prior=None,
+            animal_weight_post=None,
+            notes=None,
+            anaesthesia=expected_hp_anaesthetic,
+            headframe_type="NGC-style",
+            headframe_part_number="0160-100-10",
+            well_part_number=None,
+            well_type=None,
+        ),
+        # from list item 7
         Craniotomy.construct(
             start_date=datetime.date(2022, 12, 6),
             end_date=datetime.date(2022, 12, 6),
@@ -453,26 +493,21 @@ class Examples:
             workstation_id="SWS 3",
             notes=None,
         ),
-        FiberImplant.construct(
+        Headframe.construct(
             start_date=datetime.date(2022, 12, 6),
             end_date=datetime.date(2022, 12, 6),
-            experimenter_full_name="NSB-187",
+            experimenter_full_name="NSB",
             iacuc_protocol="2115",
             animal_weight_prior=None,
             animal_weight_post=None,
             notes=None,
-            probes=[expected_probe1],
-        ),
-        FiberImplant.construct(
-            start_date=datetime.date(2022, 12, 6),
-            end_date=datetime.date(2022, 12, 6),
-            experimenter_full_name="NSB-187",
-            iacuc_protocol="2115",
-            animal_weight_prior=None,
-            animal_weight_post=None,
-            notes=None,
-            probes=[expected_probe1, expected_probe2],
+            anaesthesia=expected_hp_anaesthetic,
+            headframe_type=None,
+            headframe_part_number=None,
+            well_part_number=None,
+            well_type=None,
         ),
+        # from list item 12
         SubjectProcedure.construct(
             start_date=datetime.date(2022, 12, 6),
             end_date=datetime.date(2022, 12, 6),
@@ -511,6 +546,9 @@ class Examples:
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
             instrument_id="Ionto #1",
             recovery_time=None,
+            injection_materials=expected_inj_materials_1,
+            injection_coordinate_reference=bregma_reference,
+            bregma_to_lambda_distance=6.1,
         ),
         # from list item 9
         Headframe.construct(
@@ -543,6 +581,7 @@ class Examples:
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
             recovery_time=30,
             workstation_id="SWS 5",
+            craniotomy_coordinates_reference=lambda_reference,
         ),
         # from list item 10
         Craniotomy.construct(
@@ -553,8 +592,8 @@ class Examples:
             animal_weight_prior=25.2,
             animal_weight_post=28.2,
             procedure_type="Craniotomy",
-            craniotomy_type="5mm",
-            craniotomy_size=5.0,
+            craniotomy_type="3mm",
+            craniotomy_size=3.0,
             craniotomy_hemisphere=None,
             craniotomy_coordinates_ml=None,
             craniotomy_coordinates_ap=None,
@@ -591,6 +630,7 @@ class Examples:
             injection_volume=500.0,
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
             recovery_time=25,
+            injection_coordinate_reference=bregma_reference,
         ),
         NanojectInjection.construct(
             start_date=None,
@@ -609,6 +649,7 @@ class Examples:
                 type="isoflurane", duration=120, level=2.5
             ),
             recovery_time=30,
+            injection_coordinate_reference=bregma_reference,
         ),
         NanojectInjection.construct(
             start_date=datetime.date(2022, 1, 3),
@@ -625,6 +666,7 @@ class Examples:
             injection_volume=600.0,
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
             recovery_time=25,
+            injection_coordinate_reference=bregma_reference,
         ),
         # from list item 14
         Headframe.construct(
@@ -658,6 +700,7 @@ class Examples:
             notes=None,
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
             recovery_time=25,
+            injection_coordinate_reference=bregma_reference,
         ),
         FiberImplant.construct(
             start_date=datetime.date(2022, 1, 3),
@@ -674,6 +717,7 @@ class Examples:
                 stereotactic_coordinate_ml=-5.2,
                 stereotactic_coordinate_dv=-2.95,
                 angle=0.0,
+                stereotactic_coordinate_reference=bregma_reference,
             ),
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
         ),
@@ -695,6 +739,7 @@ class Examples:
             alternating_current="7/7",
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
             recovery_time=25,
+            injection_coordinate_reference=bregma_reference,
         ),
         FiberImplant.construct(
             start_date=datetime.date(2022, 1, 3),
@@ -711,6 +756,7 @@ class Examples:
                 stereotactic_coordinate_ml=-0.5,
                 stereotactic_coordinate_dv=-1.05,
                 angle=0.0,
+                stereotactic_coordinate_reference=bregma_reference,
             ),
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
         ),
@@ -731,6 +777,7 @@ class Examples:
             alternating_current="7/7",
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
             recovery_time=25,
+            injection_coordinate_reference=bregma_reference,
         ),
         FiberImplant.construct(
             start_date=datetime.date(2022, 1, 3),
@@ -746,6 +793,7 @@ class Examples:
                 stereotactic_coordinate_ml=-2.2,
                 stereotactic_coordinate_dv=-1.85,
                 angle=0.0,
+                stereotactic_coordinate_reference=bregma_reference,
             ),
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
         ),
@@ -766,6 +814,7 @@ class Examples:
             alternating_current="7/7",
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
             recovery_time=25,
+            injection_coordinate_reference=bregma_reference,
         ),
         FiberImplant.construct(
             start_date=datetime.date(2022, 1, 3),
@@ -781,6 +830,7 @@ class Examples:
                 stereotactic_coordinate_ml=-2.5,
                 stereotactic_coordinate_dv=-1.8,
                 angle=0.0,
+                stereotactic_coordinate_reference=bregma_reference,
             ),
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
         ),
@@ -829,7 +879,7 @@ class Examples:
             well_part_number="0160-200-62",
             anaesthesia=Anaesthetic(type="isoflurane", duration=90, level=2.0),
         ),
-        # # from list item 13
+        # from list item 13
         SubjectProcedure.construct(
             start_date=datetime.date(2022, 1, 3),
             end_date=datetime.date(2022, 1, 3),
